/**
     File XCVM.cpp: Implementation of class CXVM
     Copyright (c) 2006 Yuanhui Xiao

     This program is free software; you can redistribute it 
     and/or modify it under the terms of the 
     GNU General Public License as published by 
     the Free Software Foundation;
     either version 2 of the License, 
     or (at your option) any later version.

     This program is distributed in the hope that it will be useful, 
     but WITHOUT ANY WARRANTY; without even the implied warranty of 
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
     the GNU General Public License for more details.

     You should have received a copy of the GNU General Public License 
     along with this program; if not, write to:

        the Free Software Foundation, Inc.
        51 Franklin St, Fifth
        Floor, Boston, MA 02110-1301, USA.

     Contact information: 
     E-mail: matyxx@langate.gsu.edu
     Adress:
        Department of Mathematics and Statistics
        Georgia State University
        750 COE, 30 Pryor Street
        Atlanta, GA 30303, USA
*/

#include "XCVM.h"

CXCVM::CXCVM(int nA, int nB)
{
   m_nA = nA; 
   m_nB = nB;

   int nGCD = GCD(); 
   m_nL = m_nA/nGCD*m_nB;

   m_nP = m_nL/m_nA;
   m_nQ = m_nL/m_nB;

   m_dCoe = (1.0/m_nP)*(1.0/m_nQ);
   m_dCoe /= (m_nA+m_nB);
   m_dCoe /= (m_nA+m_nB);
}

CXCVM::~CXCVM()
{

}

int CXCVM::ConvertToInt(double dStat) const
{
   double iStat = dStat/m_dCoe;

   // find the integer closer to iStat   

   double dCeil = ceil( iStat );
   double dFloor = floor( iStat );

  if(dFloor+dCeil < 2*iStat) 
    return static_cast<int>(dCeil );
  else
    return static_cast<int>(dFloor);
}

double CXCVM::ConvertToDouble(int iStat) const
{
   return iStat*m_dCoe; 
}

int CXCVM::GCD()
{
   int m = m_nA;
   int n = m_nB;

   while(1){

      m %= n;
      if(0 == m) return n;

      n %= m;
      if(0 == n) return m;      
   }
}

