/**

     File XCVMDist: implemenation of class CXCVMDist

     Copyright (c) 2006 Yuanhui Xiao

     This program is free software; you can redistribute it 
     and/or modify it under the terms of the 
     GNU General Public License as published by 
     the Free Software Foundation;
     either version 2 of the License, 
     or (at your option) any later version.

     This program is distributed in the hope that it will be useful, 
     but WITHOUT ANY WARRANTY; without even the implied warranty of 
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
     the GNU General Public License for more details.

     You should have received a copy of the GNU General Public License 
     along with this program; if not, write to:

        the Free Software Foundation, Inc.
        51 Franklin St, Fifth
        Floor, Boston, MA 02110-1301, USA.

     Contact information: 
     E-mail: matyxx@langate.gsu.edu
     Adress:
        Department of Mathematics and Statistics
        Georgia State University
        750 COE, 30 Pryor Street
        Atlanta, GA 30303, USA
*/


#include "XCVMDist.h"

CXCVMDist::CXCVMDist(int nA, int nB, int iFF)
          :CXCVM( (nA <= nB ? nA : nB), (nA <= nB ? nB : nA) )
{
   double* coes = new double[m_nA+1];

   // compute {m+n \choose m}

   int u, v;
   for(v=0; v<=m_nA; v++)
      coes[v] = 1.0;

   for(u=1; u<=m_nB; u++)
     for(v=1; v<=m_nA; v++)
        coes[v] += coes[v-1];

   m_dFreqSum = coes[m_nA];
   
   delete [] coes;   

   m_iFF = iFF;     

   switch( m_iFF ){

     case DIST:
        TabulateFreqFunct();
        m_functs[0].FormTails();
        break;
 
     case FFS:
        TabulateFreqFuncts();
        break;
 
     default:   // should not get here
        break; 
   }
}

CXCVMDist::~CXCVMDist()
{
   delete [] m_functs;
}

double CXCVMDist::GetCumuProb(double dStat)
{
   return 1-GetTailProb(dStat); 
}

double CXCVMDist::GetTailProb(double dStat)
{
   int iStat = ConvertToInt(dStat);
  
   if(m_iFF == FFS)
      return GetTailProb_FFs(iStat);
   else
      return GetTailProb_FF(iStat);
}

void CXCVMDist::PrintDistFunct(ostream &os)
{
   delete [] m_functs;  // destroy existing tables
   TabulateFreqFunct(); 

   int nVS = m_functs[0].GetNodes();
   int iwidthv = static_cast<int>(1+log10(static_cast<double>(nVS)));
   int iwidthzeta = static_cast<int>(1+log10(static_cast<double>(
                                      m_functs[0][nVS].GetValue())));
   int iwidthf = static_cast<int>(1+log10(m_dFreqSum));

   if(iwidthv    < 3) iwidthv = 3;
   if(iwidthzeta < 4) iwidthzeta = 4;

   if(iwidthf < 10) iwidthf = 10;
   if(iwidthf > 20) iwidthf = 20;
    
   os<<"Distribution for Cramer-von Mise Test Statistic under H0 \r\n"
     <<"m = "<<m_nA<<", n = "<<m_nB<<" \r\n\r\n";

   os<<setw(iwidthv)<<left<<"No."<<"\t"            // No. 	
     <<"      T2      "<<"\t"              // T2
     <<" Probability  "<<"\t"     // prob
     <<"  Cumu-Prob.  "<<"\t"      // Cumu. probability
     <<setw(iwidthzeta)<<right<<"Zeta"<<"\t"       // zeta 
     <<setw(iwidthf)<<right<<"Frequency"<<"\t"     // freq
     <<setw(iwidthf)<<right<<"Cumu Freq."<<"\r\n"; // Cumu. frequency

   double dProb, dCumuProb = 0;
   double dFreq, dCumuFreq = 0;

   for(int i=1; i<=nVS; i++){

         dFreq     = m_functs[0][i].GetFreq(); 
         dCumuFreq += dFreq;   
             // cumulative frequency
 
         dProb     = dFreq/m_dFreqSum;
         dCumuProb = dCumuFreq/m_dFreqSum;  
             // cumulative probability

         double T2 = ConvertToDouble(m_functs[0][i].GetValue());
             // Statistic

         // printing 
         
	 os<<setw(iwidthv)<<left<<i<<"\t";  // Sequential number

         os<<right; 

	 os<<setw(14)<<setprecision(10)<<fixed<<T2<<"\t";

         os<<setw(14);        
         if(dProb < _PF) 
            os<<setprecision(8)<<scientific;
         else            
            os<<setprecision(12)<<fixed;
	 os<<dProb<<"\t";
	 
         os<<setw(14);
         if(dCumuProb < _PF)
            os<<setprecision(8)<<scientific;
         else
            os<<setprecision(12)<<fixed;
	 os<<dCumuProb<<"\t";
	
         os<<setw(iwidthzeta)<<m_functs[0][i].GetValue()<<"\t"; // zeta
		 
         os<<setw(iwidthf);
         if(dFreq <= _FF)
           os<<setprecision(0)<<fixed;
         else
           os<<setprecision(4)<<scientific;           
         os<<dFreq<<"\t"; 

         os<<setw(iwidthf);
         if(dCumuFreq <= _FF)
           os<<setprecision(0)<<fixed;
         else
           os<<setprecision(4)<<scientific;
      	 os<<dCumuFreq<<_NEWLINE;
   }

   os<<_NEWLINE<<_NEWLINE
     <<" *** End of file *** "<<_NEWLINE;     

   // now we have the tail probability distribution

   m_iFF = DIST;
   m_functs[0].FormTails();   
}

void CXCVMDist::TabulateFreqFunct()
{
    TabulateFreqFuncts(m_nB);
    
    // final stage

    int u, v, d;

    for(u=1; u<=m_nA; u++){

       for(v=0; v<=m_nA-u; v++){

          m_functs[v] += m_functs[v+1];
          d = -m_nL + (u+v)*m_nP + v*m_nQ;  
          m_functs[v] += d*d;
       }
       m_functs[m_nA-u+1].ReSetNodes( 0 ); // release the memory
    }   

    // m_functs[0] is what we need
}

void CXCVMDist::TabulateFreqFuncts()
{
   TabulateFreqFuncts((m_nA+m_nB)/2);
}

void CXCVMDist::TabulateFreqFuncts(int iStep)
{
   m_functs = new CXFreqFunct[m_nA+1];

   int u, v, d, iAdd;

   // initialize
   iAdd = 0;
   for(v=1; v<=m_nA; v++){

      d = m_nP*v;
      iAdd += d*d;
      m_functs[v] += iAdd;
   }

   // full v-loops        
   for(u=1; u<=iStep-m_nA; u++){

      d = -m_nQ*u; 
      m_functs[0] += d*d;
      for(v=1; v<=m_nA; v++){
          
         m_functs[v] += m_functs[v-1];
         d = m_nP*v-m_nQ*u;
         m_functs[v] += d*d;
      }
   }

    // non-full v-loops
   for(u=iStep-m_nA+1; u<=iStep; u++){

      d = -m_nQ*u;
      m_functs[0] += d*d;
      for(v=1; v<=iStep-u; v++){

         m_functs[v] += m_functs[v-1];
         d = m_nP*v-m_nQ*u;
         m_functs[v] += d*d;
      }
   }
}

double CXCVMDist::GetTailProb_FF(int Q) const
{
   int i = m_functs[0].MinIndexNotLessThan(Q);

   if( i > m_functs[0].GetNodes() )
      return 0;
   else if(i < 1)
      return 1;
   else
      return m_functs[0][i].GetFreq()/m_dFreqSum; 
}

double CXCVMDist::GetTailProb_FFs(int Q) const
{
   return 0; // should not be called
}

