/**
     Class CXCVMDist: the base class modeling the distribution function of the Cramer-von Mises
                      test statistic under null hypothesis. Objects of the class can NOT be used.
                      Instead, use objects of the following subclasses.

     Subclasses: 1) CXCVMDistEQ   for the case m = n
                 2) CXCVMDistNE1  for the case m+n is even (and m != n)
                 3) CXCVMDistNE2  for the case m+n is odd  (and m != n)


     Copyright (c) 2006 Yuanhui Xiao

     This program is free software; you can redistribute it 
     and/or modify it under the terms of the 
     GNU General Public License as published by 
     the Free Software Foundation;
     either version 2 of the License, 
     or (at your option) any later version.

     This program is distributed in the hope that it will be useful, 
     but WITHOUT ANY WARRANTY; without even the implied warranty of 
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
     the GNU General Public License for more details.

     You should have received a copy of the GNU General Public License 
     along with this program; if not, write to:

        the Free Software Foundation, Inc.
        51 Franklin St, Fifth
        Floor, Boston, MA 02110-1301, USA.

     Contact information: 
     E-mail: matyxx@langate.gsu.edu
     Adress:
        Department of Mathematics and Statistics
        Georgia State University
        750 COE, 30 Pryor Street
        Atlanta, GA 30303, USA
*/

#include "XCVM.h"
#include "XFreqFunct.h"

#include <iostream>
#include <iomanip>

#ifndef CXCVMDist_H
#define CXCVMDist_H

#define _NEWLINE ("          \r\n")

#define DIST 1
#define FFS  2

// DIST: prob distribution function should be/is given
// FFS: freq functions at middle stage should be/are given

#define _PF  (1.0e-4)
#define _FF  (1.0e+7)

/*
   If a probability is large then PF, it is printed in fixed point format,
   otherwise, in scientific format when producing a probability distribution
   function

   If a frequency does not exceed FF, it is printed in fixed point format,
   otherwise, in scientific format when producing a probability distribution
   function
*/

using namespace std;

class CXCVMDist : public CXCVM{

friend ostream & operator<<(ostream &os, CXCVMDist &dist);

public:

  CXCVMDist(int nA, int nB, int iFF=FFS); // constructor
  virtual ~CXCVMDist(); 

  double GetCumuProb(double dStat); // return Prob{T2 < dStat}
  double GetTailProb(double dStat); // return Prob{T2 >= dStat}
  void   PrintDistFunct(ostream &os); // prints the distribution function

protected:

  virtual void TabulateFreqFunct();            // tabulate the probability function
  virtual void TabulateFreqFuncts();           // tabulates the tables at middle stage

  void TabulateFreqFuncts(int iStage); // tabulates the tables at stage iStage,                                               
  
  double GetTailProb_FF(int Q) const; // return the tail prob.,
                                      // the prob. dist. funct is given

  virtual double GetTailProb_FFs(int Q) const; // return the tail prob., 
                                       // the freq. functs at middle stage are given
protected:

  CXFreqFunct* m_functs;
  int          m_iFF;

  double m_dFreqSum; // sum of refequencies, which is equal to {m+n \choose m}
};
#endif
