/**
     File XCVMDistEQ.cpp: implementation of class CXCVMDistEQ

     Copyright (c) 2006 Yuanhui Xiao

     This program is free software; you can redistribute it 
     and/or modify it under the terms of the 
     GNU General Public License as published by 
     the Free Software Foundation;
     either version 2 of the License, 
     or (at your option) any later version.

     This program is distributed in the hope that it will be useful, 
     but WITHOUT ANY WARRANTY; without even the implied warranty of 
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
     the GNU General Public License for more details.

     You should have received a copy of the GNU General Public License 
     along with this program; if not, write to:

        the Free Software Foundation, Inc.
        51 Franklin St, Fifth
        Floor, Boston, MA 02110-1301, USA.

     Contact information: 
     E-mail: matyxx@langate.gsu.edu
     Adress:
        Department of Mathematics and Statistics
        Georgia State University
        750 COE, 30 Pryor Street
        Atlanta, GA 30303, USA
*/

#include "XCVMDistEQ.h"

CXCVMDistEQ::CXCVMDistEQ(int nA, int iFF) : CXCVMDist(nA, nA, iFF)
{  

}

void CXCVMDistEQ::TabulateFreqFunct()
{
   TabulateFreqFuncts();

   // final stage

   int u, v, d;

   for(u=1; u<=m_nA; u++){

     for(v=0; v<(m_nA-u)/2; v++){

        m_functs[v] += m_functs[v+1];
        d = 2*v-(m_nA-u);        
        m_functs[v] += d*d;
     }

     v = (m_nA-u)/2;
     if(1 == (m_nA+u)%2){

        m_functs[v] += m_functs[v+1];
        d = -1;
        m_functs[v] += d*d;

        m_functs[v+1].ReSetNodes( 0 ); // release memory
      }else{

         m_functs[v] *= 2;
      }        
   }    

   // m_functs[0] is what we want 
}

void CXCVMDistEQ::TabulateFreqFuncts()
{
   m_functs = new CXFreqFunct[m_nA/2+1];

   int u, v,  d;   

  // full v-loops     

   for(u=1; u<=m_nA/2; u++){

      d = -u;
      m_functs[0] += d*d;        
      for(v=1; v<u; v++){

         m_functs[v] += m_functs[v-1];
         d = v-u;
         m_functs[v] += d*d;
       }
       m_functs[u] = m_functs[u-1];
       m_functs[u] *= 2;
   }

   // non-full v-loops

   for(u=m_nA/2+1; u<=m_nA; u++){

     d = -u;      
     m_functs[0] += (d*d);

     for(v=1; v<=m_nA-u; v++){

       m_functs[v] += m_functs[v-1];
       d = v-u;
       m_functs[v] += d*d;
     }        
   } 
}

double CXCVMDistEQ::GetTailProb_FFs(int Q) const
{
   double dFreq = 0.0;
   for(int i=0; i<(m_nA+1)/2; i++){

      int d = 2*i-m_nA;
      dFreq += m_functs[i].GetConvolutionTail(Q+d*d);
   }

   dFreq *= 2;
   if(0 == m_nA%2)
     dFreq += m_functs[m_nA/2].GetConvolutionTail(Q);

   return dFreq/m_dFreqSum;
}
