/**
     File XCVMStat.cpp: Implementation of class CXCVMStat

     Copyright (c) 2006 Yuanhui Xiao

     This program is free software; you can redistribute it 
     and/or modify it under the terms of the 
     GNU General Public License as published by 
     the Free Software Foundation;
     either version 2 of the License, 
     or (at your option) any later version.

     This program is distributed in the hope that it will be useful, 
     but WITHOUT ANY WARRANTY; without even the implied warranty of 
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
     the GNU General Public License for more details.

     You should have received a copy of the GNU General Public License 
     along with this program; if not, write to:

        the Free Software Foundation, Inc.
        51 Franklin St, Fifth
        Floor, Boston, MA 02110-1301, USA.

     Contact information: 
     E-mail: matyxx@langate.gsu.edu
     Adress:
        Department of Mathematics and Statistics
        Georgia State University
        750 COE, 30 Pryor Street
        Atlanta, GA 30303, USA
*/

#include "XCVMStat.h"

CXCVMStat::CXCVMStat(int nA, int nB) : CXCVM(nA, nB)
{
   // may not be in need when indices of ranks
   // of the sample are given. However, this is not
   // the usual case. Normally, only samples are given

   m_indices = new int[m_nA+m_nB];
   m_indices--;  
}

CXCVMStat::~CXCVMStat()
{
   delete [] (++m_indices);
}

double CXCVMStat::operator()(const int* IndicesOfRanks) const
{
   return ComputeStatistic(IndicesOfRanks);
}

double CXCVMStat::operator()(const double* Sample)
{
   return ComputeStatistic(Sample);
}

double CXCVMStat::ComputeStatistic(const int* IndicesOfRanks) const
{
   int h = 0, zeta = 0;   
   for(int iRank=1; iRank<=m_nA+m_nB; iRank++){

      IndicesOfRanks[iRank] <= m_nA ? h += m_nP : h -= m_nQ; 
      zeta += h*h;
   }   

   return zeta*m_dCoe;
}

double CXCVMStat::ComputeStatistic(const double* sample) 
{	
   for(int i=1; i<=m_nA+m_nB; i++) m_indices[i] = i;
   std::sort(m_indices+1, m_indices+1+m_nA+m_nB, CXSampleRanker(sample));
   return ComputeStatistic(m_indices);
}

// Definition of the class CXSampleRanker

CXSampleRanker::CXSampleRanker(const double * x) : m_x(x)
{
}

inline bool CXSampleRanker::operator()(int i, int j) const
{
   return m_x[i] < m_x[j];
}
