/**
     Class CXCVMStat: the class for computing the Cramer-von Mises 
                     two sample test statistics

     Copyright (c) 2006 Yuanhui Xiao

     This program is free software; you can redistribute it 
     and/or modify it under the terms of the 
     GNU General Public License as published by 
     the Free Software Foundation;
     either version 2 of the License, 
     or (at your option) any later version.

     This program is distributed in the hope that it will be useful, 
     but WITHOUT ANY WARRANTY; without even the implied warranty of 
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
     the GNU General Public License for more details.

     You should have received a copy of the GNU General Public License 
     along with this program; if not, write to:

        the Free Software Foundation, Inc.
        51 Franklin St, Fifth
        Floor, Boston, MA 02110-1301, USA.

     Contact information: 
     E-mail: matyxx@langate.gsu.edu
     Adress:
        Department of Mathematics and Statistics
        Georgia State University
        750 COE, 30 Pryor Street
        Atlanta, GA 30303, USA

*/

#include "XCVM.h"

#include <algorithm>
#include <functional>
#include <iostream>

#ifndef CXCVMSTAT_H
#define CXCVMSTAT_H

#define TINY 1.0e-100

class CXCVMStat : public CXCVM {

public:

  CXCVMStat(int nA=1, int nB=1);
  virtual ~CXCVMStat();

  // Both () operators are different names for the methods
  // ComputeStatistic() given below
  double operator()(const int* IndicesOfRanks) const;
  double operator()(const double* sample);
  
  double ComputeStatistic(const int * IndicesOfRanks) const;
      // IndicesOfRanks is 1- instead of 0-based,
      // so accessing IndicesOfRanks[0] is an error.
      
  double ComputeStatistic(const double* sample); 
      // Only observed values are given
      // Note the array is one-based, see the note above
     
protected:

  int *m_indices;   // indices for ranks
};

// class CXSampleRanker: a predicate comparing indices
// based on the corresponding sample values. An auxiliary
// class for sorting data

struct CXSampleRanker{

  const double * m_x;    // sample

  CXSampleRanker(const double * x);
  bool operator()(int i, int j) const; 
    // compare the i-th and the j-th values of the sample                                          
};

#endif
