/**
     File XFreqFunct.cpp: implementation of class CXFreqFunct

     Copyright (c) 2006 Yuanhui Xiao

     This program is free software; you can redistribute it 
     and/or modify it under the terms of the 
     GNU General Public License as published by 
     the Free Software Foundation;
     either version 2 of the License, 
     or (at your option) any later version.

     This program is distributed in the hope that it will be useful, 
     but WITHOUT ANY WARRANTY; without even the implied warranty of 
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
     the GNU General Public License for more details.

     You should have received a copy of the GNU General Public License 
     along with this program; if not, write to:

        the Free Software Foundation, Inc.
        51 Franklin St, Fifth
        Floor, Boston, MA 02110-1301, USA.

     Contact information: 
     E-mail: matyxx@langate.gsu.edu
     Adress:
        Department of Mathematics and Statistics
        Georgia State University
        750 COE, 30 Pryor Street
        Atlanta, GA 30303, USA
*/

#include "XFreqFunct.h"

CXFreqFunct::CXFreqFunct(int nodes)
{
   m_nNodes = nodes;
   m_nodes = new CXNode[m_nNodes];
   m_nodes--;
}

CXFreqFunct::CXFreqFunct(const CXFreqFunct & FreqFunct)
{
   m_nNodes = FreqFunct.m_nNodes;
   m_nodes = new CXNode[m_nNodes];   
   m_nodes--;

   for(int i=1; i<=m_nNodes; i++)
      m_nodes[i] = FreqFunct.m_nodes[i];   
}

CXFreqFunct::~CXFreqFunct()
{
   delete [] (++m_nodes);
}

CXFreqFunct::CXNode::CXNode()
{
   m_iV = 0;
   m_dF = 1;
}

int CXFreqFunct::CXNode::GetValue() const
{
   return m_iV;
}

double CXFreqFunct::CXNode::GetFreq() const
{
   return m_dF;
}

inline const CXFreqFunct::CXNode & CXFreqFunct::CXNode::operator+=(int id2)
{
   m_iV += id2; 
   return *this;
}

inline CXFreqFunct::CXNode & CXFreqFunct::CXNode::operator+=(const CXNode &node)
{
   m_dF += node.m_dF; 
   return *this;
}

inline const CXFreqFunct::CXNode & CXFreqFunct::CXNode::operator*=(int iNum)
{
   m_dF *= iNum;
   return *this;
}

inline bool CXFreqFunct::CXNode::operator==(const CXNode &Node) const
{
   return (m_iV == Node.m_iV);  
}

inline bool CXFreqFunct::CXNode::operator<(const CXNode &Node) const
{
   return (m_iV < Node.m_iV);  
}

inline bool CXFreqFunct::CXNode::operator==(const int Node) const
{
   return m_iV == Node;  
}

inline bool CXFreqFunct::CXNode::operator<=(const int Node) const
{
   return m_iV <= Node;
}

inline bool CXFreqFunct::CXNode::operator>=(const int Node) const
{
   return m_iV >= Node;  
}

inline bool CXFreqFunct::CXNode::operator<(const int Node) const
{
   return m_iV < Node;  
}

inline CXFreqFunct::iterator CXFreqFunct::begin()
{
   return m_nodes+1;
}

inline CXFreqFunct::iterator CXFreqFunct::end()
{
   return m_nodes+1+m_nNodes;
}

inline CXFreqFunct::const_iterator CXFreqFunct::begin() const
{
   return m_nodes+1;
}
   
inline CXFreqFunct::const_iterator CXFreqFunct::end() const
{
   return m_nodes+1+m_nNodes;
}

CXFreqFunct::reference CXFreqFunct::operator[](int i) 
{
   return m_nodes[i];
}

CXFreqFunct::const_reference CXFreqFunct::operator[](int i) const
{
   return m_nodes[i];
}

inline CXFreqFunct::reference CXFreqFunct::GetAt(int i) 
{
   return m_nodes[i];
}

inline CXFreqFunct::const_reference CXFreqFunct::GetAt(int i) const
{
   return m_nodes[i];
}

const CXFreqFunct &CXFreqFunct::operator=(const CXFreqFunct &FreqFunct)
{  
   if(this != &FreqFunct){

      if(m_nNodes != FreqFunct.m_nNodes){

         delete [] (++m_nodes);
         m_nNodes = FreqFunct.m_nNodes;
         m_nodes = new CXNode[m_nNodes];         
         m_nodes--;
      }

      for(int i=1; i<=m_nNodes; i++)
        m_nodes[i] = FreqFunct.m_nodes[i];
   }
   return *this;
}

const CXFreqFunct &CXFreqFunct::operator+=(const CXFreqFunct &ff) 
{
   CXFreqFunct Temp( GetNodes() + ff.GetNodes() );
   std::merge(begin(), end(), ff.begin(), ff.end(), Temp.begin());
   Temp.MergeTo(*this);

   return *this;
}

const CXFreqFunct &CXFreqFunct::operator*=(int iF)
{
   for(int i=1; i<=m_nNodes; i++)
      m_nodes[i] *= iF;  

   return *this;
}

const CXFreqFunct &CXFreqFunct::operator+=(int iV)
{
  for(int i=1; i<=m_nNodes; i++)
     m_nodes[i] += iV;
   
  return *this;
}

inline int CXFreqFunct::GetNodes() const
{
   return m_nNodes;
}

void CXFreqFunct::ReSetNodes(int nNewNodes)
{
  if(m_nNodes != nNewNodes){

     delete [] (++m_nodes);
     m_nNodes = (nNewNodes <= 0 ? 1 : nNewNodes);
     m_nodes = new CXNode[m_nNodes];
     m_nodes--;
  }
  m_nNodes = nNewNodes;
}

int CXFreqFunct::MinIndexNotLessThan(int Q, int LB, int UB) const
{
    if(m_nodes[LB] >= Q){

	return LB;

    }else if(m_nodes[UB] < Q){
  
        return UB+1;

    }else{  // binary search

	int M, L=LB, U = UB;
	while(U-L > 1){
			
	   M = (L+U)/2;
           if(m_nodes[M] == Q)
              return M;
	   else if(m_nodes[M] <= Q)
	      L = M;
	   else
	      U = M;
	}

	return (m_nodes[U]<Q ? U+1 : U);
    }
}

int CXFreqFunct::MinIndexNotLessThan(int Q) const
{
    return MinIndexNotLessThan(Q, 1, m_nNodes);
}

double CXFreqFunct::SumFreqs(int LB, int UB) const
{
    double dSum = 0.0;
    for(int i=LB; i<=UB; i++)
       dSum += m_nodes[i].GetFreq();

    return dSum;
}

double CXFreqFunct::GetConvolutionTail(int Q) const
{
    double dFreq = 0.0;

    int S0 = MinIndexNotLessThan( Q-m_nodes[m_nNodes].GetValue() );
    int LB, UB;
    double dSumFreq = 0.0;

    LB  = m_nNodes+1;
    for(int i=S0; i<=m_nNodes; i++){

	UB = LB-1;
	LB = MinIndexNotLessThan(Q-m_nodes[i].GetValue(), S0, UB );
	dSumFreq += SumFreqs(LB, UB);
	dFreq += dSumFreq*m_nodes[i].GetFreq();
	if(LB == S0){

	   dFreq += dSumFreq*SumFreqs(i+1, m_nNodes);
	   break;
	}
     }

     return dFreq;
}

double CXFreqFunct::GetConvolutionTail(const CXFreqFunct &ff, int Q) const
{
   double dFreq = 0;

   int nNodes_x = m_nNodes;
   int nNodes_y = ff.GetNodes();

   int Sx = MinIndexNotLessThan( Q-ff[nNodes_y].GetValue() );     
   int Sy = ff.MinIndexNotLessThan( Q-m_nodes[nNodes_x].GetValue() );
   int LB, UB;
   double dSumFreq = 0;
   if( nNodes_x-Sx <= nNodes_y-Sy ){

      LB = nNodes_y+1;      
      for(int i=Sx; i<=nNodes_x; i++){
         
         UB = LB-1;
         LB = ff.MinIndexNotLessThan(Q-m_nodes[i].GetValue(), Sy, UB); 
         dSumFreq += ff.SumFreqs(LB, UB);
         dFreq += dSumFreq*m_nodes[i].GetFreq();
         if(LB == Sy){
       
           dFreq += dSumFreq*SumFreqs(i+1, nNodes_x);
           break;
         }
      }      
   }else{

      LB = nNodes_x+1;      
      for(int i=Sy; i<=nNodes_y; i++){

         UB = LB-1;
         LB = MinIndexNotLessThan( Q-ff[i].GetValue(), Sx, UB); 		 
         dSumFreq += SumFreqs(LB, UB);
         dFreq += dSumFreq*ff[i].GetFreq();
         if(LB == Sx){
       
           dFreq += dSumFreq*ff.SumFreqs(i+1, nNodes_y);
           break;
         }
      }   
   }

   return dFreq;
}

void CXFreqFunct::FormCumus(int iLast) 
{
   if(iLast == 0)
      iLast = m_nNodes;

   for(int i=2; i<=iLast; i++)
      m_nodes[i].m_dF += m_nodes[i-1].m_dF;   
}

void CXFreqFunct::FormTails(int iFirst)
{
   for(int i=m_nNodes-1; i>=iFirst; i--)
      m_nodes[i].m_dF += m_nodes[i+1].m_dF;
}

void CXFreqFunct::MergeTo(CXFreqFunct &Merged)
{
   int n = GetNodes();
   CXFreqFunct temp(n);

   temp[1] = GetAt(1);
   int i, iM=1;
   for(i=2; i<=n; i++){

     if(GetAt(i) == temp[iM])
       temp[iM] += GetAt(i);
     else
       temp[++iM] = GetAt(i);     
   }   

   Merged.ReSetNodes(iM);
   for(i=1; i<=iM; i++)
     Merged[i] = temp[i];
}

