/**
     class CXFreqFunct --- Frequency tabl or frequency function

     Copyright (c) 2006 Yuanhui Xiao

     This program is free software; you can redistribute it 
     and/or modify it under the terms of the 
     GNU General Public License as published by 
     the Free Software Foundation;
     either version 2 of the License, 
     or (at your option) any later version.

     This program is distributed in the hope that it will be useful, 
     but WITHOUT ANY WARRANTY; without even the implied warranty of 
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
     the GNU General Public License for more details.

     You should have received a copy of the GNU General Public License 
     along with this program; if not, write to:

        the Free Software Foundation, Inc.
        51 Franklin St, Fifth
        Floor, Boston, MA 02110-1301, USA.

     Contact information: 
     E-mail: matyxx@langate.gsu.edu
     Adress:
        Department of Mathematics and Statistics
        Georgia State University
        750 COE, 30 Pryor Street
        Atlanta, GA 30303, USA
*/

#ifndef CXFreqFunct_H
#define CXFreqFunct_H

#include <cmath>
#include <iostream>
#include <iomanip>
#include <algorithm>

// Declaration of the class CXFreqFunct

class CXFreqFunct{

public:

   explicit CXFreqFunct(int areas=1);
   CXFreqFunct(const CXFreqFunct & FreqFuncte);
   ~CXFreqFunct();

   class CXNode{ // the pair of an integer and its frequency

	 friend class CXFreqFunct;

     public:

      CXNode();

      int GetValue() const;
      double GetFreq() const;	  
      
      const CXNode &operator+=(int iV); // adds iV to the value of the pair  

      CXNode & operator+=(const CXNode &Node); // adds freqs, summe same value
      const CXNode &operator*=(int iF); // the freq is multiplied by iF

	      // The methods below only compare the values, no matter what the freqs are.
      bool operator==(const CXNode & Node) const;     
      bool operator<( const CXNode & Node) const;    
      
      bool operator==(const int Node) const; 
      bool operator<=(const int Node) const;
      bool operator>=(const int Node) const;
      bool operator<( const int Node) const;         

	 protected:

      int    m_iV; // the integer or value
      double m_dF; // the frequency, which may be huge.
                   // So it is represented as double in
                   // instead of int.
   };

   typedef CXFreqFunct::CXNode* iterator;
   typedef const CXFreqFunct::CXNode* const_iterator;

   typedef CXFreqFunct::CXNode& reference;
   typedef const CXFreqFunct::CXNode& const_reference;

   iterator begin();
   iterator end();

   const_iterator begin() const;
   const_iterator end() const;

   reference operator[](int i);  // accesses the i-th pair or node
   const_reference operator[](int i) const; 

   reference GetAt(int i);  // accesses the i-th pair or node
   const_reference GetAt(int i) const; 

   const CXFreqFunct & operator=(const CXFreqFunct &);  // assignement  
   const CXFreqFunct & operator+=(const CXFreqFunct &); // sum of frequency functions 
   const CXFreqFunct & operator*=(int iF); // each freq is multiplied by iF
   const CXFreqFunct & operator+=(int iV); // shift the freq. funct or table

   int GetNodes() const;   // gets the number of pairs or nodes
   void ReSetNodes(int nNewNodes); // reset the number of nodes or pair in the table

   int MinIndexNotLessThan(int Q) const; // finds the smallest index of the pair or node
                                         // whose value is not less than Q
   int MinIndexNotLessThan(int Q, int LB, int UB) const;
                                           // Same as above, but the search is restricted
                                           // in the range (LB, UB)
   double SumFreqs(int LB, int UB) const;  // computes the sum of the frequencies 
                                           // of the nodes or pairs whose values fall in
                                           // the range (LB, UB)
   double GetConvolutionTail(int Q) const; // finds the sum of the frequencies of the nodes
                                           // or pairs in the convolution of the table with
                                           // itself whose values are not less than Q.                                          
   double GetConvolutionTail(const CXFreqFunct &ff, int Q) const;
                                           // Same as above, but in this case the two tables
                                           // are different.                                           
   void FormCumus(int iLast=0);  // forms the cumulative frequency table
   void FormTails(int iFirst=1); // forms the tail frequency table

protected:

   void MergeTo(CXFreqFunct &merged);

protected:

   CXNode* m_nodes; // the array holding the nodes or pairs. It is 1-based. 
   int m_nNodes;    // number of nodes
};

#endif
