/**
     File XNumberCheckers.cpp: implementation of functions that check input strings

     Copyright (c) 2006 Yuanhui Xiao

     This program is free software; you can redistribute it 
     and/or modify it under the terms of the 
     GNU General Public License as published by 
     the Free Software Foundation;
     either version 2 of the License, 
     or (at your option) any later version.

     This program is distributed in the hope that it will be useful, 
     but WITHOUT ANY WARRANTY; without even the implied warranty of 
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
     the GNU General Public License for more details.

     You should have received a copy of the GNU General Public License 
     along with this program; if not, write to:

        the Free Software Foundation, Inc.
        51 Franklin St, Fifth
        Floor, Boston, MA 02110-1301, USA.

     Contact information: 
     E-mail: matyxx@langate.gsu.edu
     Adress:
        Department of Mathematics and Statistics
        Georgia State University
        750 COE, 30 Pryor Street
        Atlanta, GA 30303, USA

*/

#include "XNumberCheckers.h"

bool IsUnsigned(char *str)
{
   if(str == NULL)
      return false; 

   int nStrLen = strlen(str);   

   for(int i=0; i<nStrLen; i++)
     if( !isdigit(str[i]) )
        return false;      

   return true; 
}

bool IsInteger(char *str)
{
   if(str == NULL)
      return false; 

   int nStrLen = strlen(str);

   // check length

   if(nStrLen == 0)
     return false;
   
   if(nStrLen == 1)
     return (isdigit(str[0]) == 0 ? false : true);

   if(str[0] == '+' || str[0] == '-' || isdigit(str[0]) )    
      return IsUnsigned( &str[1] );
   else
      return false;
}

bool IsDouble(char *str)
{ 
   // check whether there are such patterns: "..", "ee", "EE", "eE", "Ee"
   // "e.", "E."

    char * dot = strchr(str, '.');
    if(dot != NULL){

      if(dot[1] == '.')
        return false;
    }

    char * ee1 = strchr(str, 'e');
    if(ee1 != NULL){

      if(ee1[1] == 'e' || ee1[1] == 'E' || ee1[1] == '.')
        return false;
    }

    char * EE2 = strchr(str, 'E');
    if(EE2 != NULL){

      if(EE2[1] == 'E' || EE2[1] == 'e' || EE2[1] == '.')
         return false;
   }
 
   int nStrLen = strlen(str);
   char *cpy  =new char[nStrLen+1];
   strcpy(cpy, str);

   char lastcpy = cpy[strlen(cpy)-1]; // last letter of cpy   
   char * tok1 = strtok(cpy,  "eE");    
   char * tok2 = strtok(NULL, "eE");  
   char * tok3 = strtok(NULL, "eE");  

   if(tok3 != NULL)
     return false;
   
   if( tok2 != NULL ){
      if( !IsInteger(tok2) || lastcpy == 'e' || lastcpy == 'E')
          return false;
   }

   char lasttok1 = tok1[strlen(tok1)-1]; // last letter of tok1
   char * tok11 = strtok(tok1, ".");
   char * tok12 = strtok(NULL, ".");
   char * tok13 = strtok(NULL, ".");

   if( tok13 != NULL )
      return false;

   if( tok12 != NULL )
      if( !IsUnsigned(tok12) || lasttok1 == '.')
          return false;
  
   return IsInteger(tok11);
}
